@@ -8,7 +8,7 @@
 #include <errno.h>
 #include <assert.h>
 
-#include "atm.h"
+#include <atm.h>
 #include <linux/atmsvc.h>
 #include <linux/atmdev.h>
 
@@ -45,7 +45,9 @@ static int send_setup(SOCKET *sock)
     q_create(&dsc,q_buffer,MAX_Q_MSG);
     local = NULL;
     error = 0;
-    if (atmsvc_addr_in_use(sock->local)) {
+    sock->sig = route_remote(&sock->remote);
+    if (sock->sig && sock->sig->mode == sm_switch) local = &sock->local;
+    if (atmsvc_addr_in_use(sock->local) && !local) {
 	sock->sig = route_local(&sock->local);
 	if (sock->sig) local = &sock->local;
 	else {
@@ -54,10 +56,9 @@ static int send_setup(SOCKET *sock)
 	}
     }
     else {
-	sock->sig = route_remote(&sock->remote);
 	if (!sock->sig) error = -EHOSTUNREACH;
 	else {
-	    local = get_local(sock->sig);
+	    if (!local) local = get_local(sock->sig);
 	    if (!local) local = get_local(NULL);
 	    if (local) sock->local = *local;
 	    else {
@@ -561,7 +562,7 @@ void from_kernel(struct atmsvc_msg *msg,int size)
 	 */
 	return;
     }
-    if (kptr_eq(&msg->listen_vcc,&kptr_null) && (msg->type == as_accept ||
+    if (!kptr_eq(&msg->listen_vcc,&kptr_null) && (msg->type == as_accept ||
       msg->type == as_reject || msg->type == as_identify)) {
 	dispatcher = dispatch_listen;
 	for (curr = sockets; curr; curr = curr->next)
